# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import enum

import winrt.system
from winrt._winrt_windows_graphics_display_core import (
    HdmiDisplayHdr2086Metadata,
    HdmiDisplayInformation,
    HdmiDisplayMode,
)

__all__ = [
    "HdmiDisplayColorSpace",
    "HdmiDisplayHdrOption",
    "HdmiDisplayPixelEncoding",
    "HdmiDisplayHdr2086Metadata",
    "HdmiDisplayInformation",
    "HdmiDisplayMode",
]

class HdmiDisplayColorSpace(enum.IntEnum):
    RGB_LIMITED = 0
    RGB_FULL = 1
    BT2020 = 2
    BT709 = 3

class HdmiDisplayHdrOption(enum.IntEnum):
    NONE = 0
    EOTF_SDR = 1
    EOTF2084 = 2
    DOLBY_VISION_LOW_LATENCY = 3

class HdmiDisplayPixelEncoding(enum.IntEnum):
    RGB444 = 0
    YCC444 = 1
    YCC422 = 2
    YCC420 = 3

